import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { IconByName } from '@pikas-ui/icons';
import { styled, useTheme, } from '@pikas-ui/styles';
import { forwardRef, useEffect, useRef, useState, } from 'react';
import * as LabelPrimitive from '@radix-ui/react-label';
import { Description, Label, TextError } from '@pikas-ui/text';
import { Tooltip } from '@pikas-ui/tooltip';
import { Color } from '@pikas-utils/color';
import { MultiInputFieldBadge } from './badge/index.js';
import { useMergedRef } from '@pikas-utils/hooks';
const Container = styled('div', {
    display: 'flex',
    flexDirection: 'column',
    userSelect: 'none',
});
const InputContainer = styled('div', {
    width: '100%',
    overflow: 'hidden',
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
    borderStyle: 'solid',
    variants: {
        focus: {
            true: {
                outline: 'solid',
                outlineColor: '$PRIMARY',
                outlineWidth: 2,
            },
        },
        gap: {
            xs: {
                customColumnGap: 2,
            },
            sm: {
                customColumnGap: 4,
            },
            md: {
                customColumnGap: 8,
            },
            lg: {
                customColumnGap: 16,
            },
            xl: {
                customColumnGap: 32,
            },
        },
    },
});
const InputContent = styled('div', {
    display: 'flex',
    alignItems: 'center',
    flexWrap: 'wrap',
    width: '100%',
    variants: {
        padding: {
            xs: {
                padding: '4px 8px',
            },
            sm: {
                padding: '6px 12px',
            },
            md: {
                padding: '8px 16px',
            },
            lg: {
                padding: '12px 24px',
            },
            xl: {
                padding: '16px 32px',
            },
        },
        gap: {
            xs: {
                customGap: 2,
            },
            sm: {
                customGap: 4,
            },
            md: {
                customGap: 8,
            },
            lg: {
                customGap: 16,
            },
            xl: {
                customGap: 32,
            },
        },
    },
});
const Input = styled('input', {
    width: 'initial',
    minWidth: 120,
    outline: 'none',
    fontSize: '$EM-SMALL',
    border: 'none',
    fontFamily: '$roboto',
    backgroundColor: '$TRANSPARENT',
    flex: 1,
});
const LeftContainer = styled(LabelPrimitive.Root, {
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
    variants: {
        padding: {
            xs: {
                padding: 2,
            },
            sm: {
                padding: 4,
            },
            md: {
                padding: 8,
            },
            lg: {
                padding: 16,
            },
            xl: {
                padding: 32,
            },
        },
    },
});
const RightContainer = styled(LabelPrimitive.Root, {
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'center',
    variants: {
        padding: {
            xs: {
                padding: 2,
            },
            sm: {
                padding: 4,
            },
            md: {
                padding: 8,
            },
            lg: {
                padding: 16,
            },
            xl: {
                padding: 32,
            },
        },
    },
});
const LabelContainer = styled('div', {
    display: 'flex',
    marginBottom: 4,
});
const Required = styled('div', {
    color: '$WARNING',
    marginLeft: 4,
});
export const multiInputFieldType = {
    color: true,
    date: true,
    'datetime-local': true,
    email: true,
    hidden: true,
    month: true,
    number: true,
    password: true,
    search: true,
    tel: true,
    text: true,
    time: true,
    url: true,
    week: true,
};
export const multiInputFieldPadding = {
    xs: true,
    sm: true,
    md: true,
    lg: true,
    xl: true,
};
export const multiInputFieldGap = {
    xs: true,
    sm: true,
    md: true,
    lg: true,
    xl: true,
};
const getColor = ({ colorName, colorHex, backgroundColorName, }) => {
    const theme = useTheme();
    if (colorHex) {
        return colorHex;
    }
    else if (colorName) {
        return `$${colorName}`;
    }
    else if (theme) {
        return new Color(theme.colors[backgroundColorName ?? 'WHITE'].value).getContrast();
    }
    else {
        return '';
    }
};
export const MultiInputField = forwardRef(({ id, type = 'text', onChange, boxShadow = 'DIMINUTION_1', borderRadius = 'md', padding = 'md', fontSize = 'EM-MEDIUM', textError, label, css, borderColorName = 'TRANSPARENT', borderWidth = 0, autoComplete, min, max, LeftIcon, RightIcon, leftChildren, rightChildren, backgroundColorName = 'GRAY_LIGHTEST_1', outline = true, description, gap = 'sm', width = '100%', maxWidth = '100%', minWidth, backgroundColorHex, borderColorHex, colorName, colorHex, placeholderColorName = 'BLACK_LIGHT', placeholderColorHex, leftIconColorName, leftIconColorHex, rightIconColorName, rightIconColorHex, leftIconSize, rightIconSize, required, info, disabled = false, data, defaultValues = [], ...props }, ref) => {
    const refInput = useRef(null);
    const multiRef = useMergedRef(ref, refInput);
    const [focus, setFocus] = useState(false);
    const [currentValues, setCurrentValues] = useState(defaultValues);
    const handleDeleteValue = (index) => {
        setCurrentValues((currentState) => {
            const copy = [...currentState];
            copy.splice(index, 1);
            return copy;
        });
    };
    const handleRemoveLastValue = () => {
        setCurrentValues((currentState) => {
            const copy = [...currentState];
            copy.pop();
            return copy;
        });
    };
    const handleKeyUp = (e) => {
        const inputValue = e.currentTarget.value;
        if (type === 'number' && refInput.current) {
            const value = parseInt(inputValue);
            if (max !== undefined && value > max) {
                refInput.current.value = `${max}`;
            }
            else if (min !== undefined && value < min) {
                refInput.current.value = `${min}`;
            }
        }
        if (e.key === 'Enter' && !!e.currentTarget.value) {
            setCurrentValues((currentState) => [...currentState, inputValue]);
            e.currentTarget.value = '';
        }
    };
    const handleKeyDown = (e) => {
        if (e.key === 'Backspace' &&
            !e.currentTarget.value &&
            currentValues.length) {
            e.preventDefault();
            e.currentTarget.value = currentValues[currentValues.length - 1];
            handleRemoveLastValue();
        }
    };
    useEffect(() => {
        onChange?.(currentValues);
    }, [currentValues]);
    return (_jsxs(Container, { css: {
            fontSize: `$${fontSize}`,
            width,
            maxWidth,
            minWidth,
            opacity: disabled ? 0.5 : 1,
            cursor: disabled ? 'not-allowed' : undefined,
            '& > *': {
                pointerEvents: disabled ? 'none' : undefined,
            },
            ...css?.container,
        }, children: [label ? (_jsxs(LabelContainer, { children: [_jsx(Label, { htmlFor: id, css: css?.label, children: label }), required ? _jsx(Required, { css: css?.required, children: "*" }) : null, info ? (_jsx(Tooltip, { content: info, css: css?.infoTooltip, children: _jsx(IconByName, { name: "bx:info-circle", colorName: "BLACK_LIGHT", css: {
                                container: {
                                    marginLeft: 4,
                                    ...css?.infoIcon?.container,
                                },
                                svg: {
                                    ...css?.infoIcon?.svg,
                                },
                            } }) })) : null] })) : null, description ? (_jsx(Description, { css: {
                    marginBottom: 4,
                    ...css?.description,
                }, children: description })) : null, _jsxs(InputContainer, { focus: outline ? focus : undefined, gap: gap, onBlur: () => setFocus(false), css: {
                    br: borderRadius,
                    borderColor: borderColorHex ?? `$${borderColorName}`,
                    backgroundColor: backgroundColorHex ?? `$${backgroundColorName}`,
                    boxShadow: `$${boxShadow}`,
                    borderWidth,
                    ...css?.inputContainer,
                }, children: [leftChildren && (_jsx(LeftContainer, { htmlFor: id, padding: padding, css: {
                            ...(id && {
                                cursor: 'pointer',
                            }),
                            ...css?.left,
                        }, children: leftChildren })), LeftIcon && (_jsx(LeftContainer, { htmlFor: id, padding: padding, css: {
                            ...(id && {
                                cursor: 'pointer',
                            }),
                            ...css?.left,
                        }, children: _jsx(LeftIcon, { size: leftIconSize ?? '1em', colorHex: getColor({
                                colorName: leftIconColorName ?? colorName,
                                colorHex: leftIconColorHex ?? colorHex,
                                backgroundColorName,
                            }), css: css?.leftIcon }) })), _jsxs(InputContent, { padding: padding, gap: gap, css: css?.inputContent, children: [currentValues.map((item, index) => (_jsx(MultiInputFieldBadge, { index: index, value: item, onDelete: handleDeleteValue, css: css?.badge }, index))), _jsx(Input, { ref: multiRef, id: id, type: type, onKeyUp: handleKeyUp, onKeyDown: handleKeyDown, autoComplete: autoComplete, min: min, max: max, onFocus: () => setFocus(true), disabled: disabled, required: required, css: {
                                    color: getColor({ colorName, colorHex, backgroundColorName }),
                                    '&::placeholder': {
                                        color: getColor({
                                            colorName: placeholderColorName,
                                            colorHex: placeholderColorHex,
                                            backgroundColorName,
                                        }),
                                    },
                                    ...css?.input,
                                }, ...props, ...data })] }), RightIcon && (_jsx(RightContainer, { htmlFor: id, padding: padding, css: {
                            ...(id && {
                                cursor: 'pointer',
                            }),
                            ...css?.right,
                        }, children: _jsx(RightIcon, { size: rightIconSize ?? '1em', colorHex: getColor({
                                colorName: rightIconColorName ?? colorName,
                                colorHex: rightIconColorHex ?? colorHex,
                                backgroundColorName,
                            }), css: css?.rightIcon }) })), rightChildren && (_jsx(RightContainer, { htmlFor: id, padding: padding, css: {
                            ...(id && {
                                cursor: 'pointer',
                            }),
                            ...css?.right,
                        }, children: rightChildren }))] }), textError && (_jsx(TextError, { css: { marginTop: 5, ...css?.textError }, children: textError }))] }));
});
MultiInputField.displayName = 'MultiInputField';
